/**
 *  Copyright notice
 *  
 *  This file is part of the Processing sketch `Stimulated Emission' 
 *  http://www.gwoptics.org/processing/stimulated_emission/
 *
 *  Copyright (C) 2009 onwards Mengyao Wang and Andreas Freise
 *  based on an idea by Emil Schreiber.
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
 *  MA  02110-1301, USA.
 */

class EmbeddedSketch extends PApplet implements PConstants {
  PApplet _parent;  
  Laser laser;

  int sketch_width;
  int sketch_height;

  int x1;
  int x2;
  int y1;
  int dradius;
  float pl;
  float Rm2;
  float curvature;

  int number_of_atoms=100;

  PFont font;

  //The constructor must contain one arguement that contains a reference the parent/main
  //sketch. There is many reason why this must be done, the most important is that embedded
  //sketches cant load images you must call _parent.loadImage("blah.jpg").
  EmbeddedSketch(PApplet parent, int t_sketch_width, int t_sketch_height, int t_precision) {
    if(parent == null)
      throw new RuntimeException("Parent object cannot be null");
    _parent = parent;
    paused=false;
    sketch_width=t_sketch_width;
    sketch_height=t_sketch_height;
    precision = t_precision;
  }

  public void setup() {
    size(sketch_width,sketch_height,P2D);
    dradius=800;
    x1=laser_x1+dradius/2;
    x2=laser_x2-dradius/2;
    y1= laser_y1+(laser_y2-laser_y1)/2;
    //frameRate(9999);
    noLoop();    

    font = createFont("Verdana",14,true);

    laser = new Laser(this);
    slot = new float [precision];


    //-------------------------------------------------------
    // initialise atoms
    for (int i = 0; i < number_of_atoms; i++) {
      laser.addAtom(new Atom(this,1));
    }


    for (int i = 0; i < nlookup; i++) {
      coslookup[i] = cos(i/idx_scale)*cosamplitude;
      sinlookup[i] = sin(i/idx_scale)*cosamplitude;
    }
  }

  public void draw() {
    if (paused==false) {
      background(165,170,177);
      //smooth();
      pump_draw();

      // drawing aperture
      strokeWeight(5);
      stroke(0);  
      line(laser_x2+10,laser_y1,laser_x2+10,laser_y1-block);   
      line(laser_x2+10,laser_y2,laser_x2+10,laser_y2+block);   

      //drawing the laser `tube' and mirrors
      strokeWeight(2);
      stroke(90);  
      line(laser_x1-10,laser_y1,laser_x2+10,laser_y1);
      line(laser_x1-10,laser_y2,laser_x2+10,laser_y2);
      fill(0,0,0,0);
      stroke(100,150,200);    
      arc(x1-14, y1, dradius, dradius, PI-0.12, PI+0.12);
      arc(x2+14, y1, dradius,dradius, -0.12, 0.12);
      noSmooth();

      //Photons number in different slots representing as changing amplitudes and filling coloured slim rectangle
      float dsc2=-screenmax/slotmax;
      float csc2=255/sqrt(slotmax);

      noStroke();
      for (int i=avg_length; i<precision-avg_length; i++) {
        float savg=0.0;
        for (int j=-avg_length;j<=avg_length;j++) {
          savg=savg+slot[i+j];
        }
        savg=savg/(2*avg_length);
        fill(sqrt(savg+0.1)*csc2);
        int box_height=round(sketch_height/(precision-2*avg_length));
        rect(detector_x,6+(i-avg_length)*box_height,detector_width,box_height);
      }

      laser.run();
      laser.pump();

      //drawing introduction labels
      strokeWeight(1);
      stroke(0);
      line(laser_x1+50,laser_y1-40,laser_x1+70,laser_y1-70);
      writeLabels("Pump",laser_x1+80,laser_y1-70);

      line(laser_x2+10,laser_y1-30,laser_x2+35,laser_y1-65);
      writeLabels("Block",laser_x2+40,laser_y1-70);

      line(laser_x2+12,laser_y1+20,laser_x2+35,laser_y1-25);
      writeLabels("Partially Reflective Mirror",laser_x2+40,laser_y1-30);      

      line(laser_x2,laser_y2-30,laser_x2+45,laser_y2+50);
      writeLabels("Cavity",laser_x2+50,laser_y2+70);

      line(display_width-2*sketch_xoffset,laser_y1-77,display_width-5*sketch_xoffset,laser_y1-75);
      writeLabels("Detector",display_width-2*sketch_xoffset-100,laser_y1-70);
    }
  }

  void setM2Reflectivity(float Reflectivity_m2) {
    Rm2=Reflectivity_m2;
  }


  void setCurvature(float mirror_curvature) {
    curvature=mirror_curvature;
  }


  float getM2Reflectivity() {
    return Rm2;
  }


  void setLaserPump(float Pump_laser) {
    pl=Pump_laser;
    laser.setLaserPump(Pump_laser);
  }



  void em_pause() {
    paused = true;
  }

  void em_play() {
    paused = false;
  }

  void pump_draw() {
    noStroke();
    fill(231,92,231,pl*55);
    ellipse(laser_x1+150, laser_y1-30,1.2*(laser_x2-laser_x1),40);
    fill(231,92,231,pl*255);
    ellipse(laser_x1+150, laser_y1-30,laser_x2-laser_x1,20);
    fill(255,192,251,pl*255);
    ellipse(laser_x1+150, laser_y1-30,0.9*(laser_x2-laser_x1),12);
    fill(255,255,255,pl*255);
    ellipse(laser_x1+150, laser_y1-30,0.7*(laser_x2-laser_x1),2);

    fill(231,92,231,pl*55);
    ellipse(laser_x1+150, laser_y2+30,1.2*(laser_x2-laser_x1),40);
    fill(231,92,231,pl*255);
    ellipse(laser_x1+150, laser_y2+30,laser_x2-laser_x1,20);
    fill(255,192,251,pl*255);
    ellipse(laser_x1+150, laser_y2+30,0.9*(laser_x2-laser_x1),12);
    fill(255,255,255,pl*255);
    ellipse(laser_x1+150, laser_y2+30,0.7*(laser_x2-laser_x1),2);
  }

  void writeLabels(String label, int x_position, int y_position) {
    fill(0);
    textFont(font,14);
    text(label,x_position,y_position);
  }



  public void keyPressed() {		
    if(key == ' ') {
      if (paused==true) {
        unpause();
      }   
      else {
        pause();
      }
    }
  }
}



















