/**
 *  Copyright notice
 *  
 *  This file is part of the Processing sketch `Stimulated Emission' 
 *  http://www.gwoptics.org/processing/stimulated_emission/
 *
 *  Copyright (C) 2009 onwards Mengyao Wang and Andreas Freise
 *  based on an idea by Emil Schreiber.
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
 *  MA  02110-1301, USA.
 */

public class Laser {
  ArrayList atoms;
  ArrayList photons;  
  PApplet _parent;    
  int stimulation_radius=5;
  float decay_propability=0.03;
  float pl;


  public Laser(PApplet parent) {
    _parent=parent;
    atoms = new ArrayList();
    photons = new ArrayList();
    pl=0;
  }

  void run() {
    for (int i = 0; i < atoms.size(); i++) {
      Atom a = (Atom) atoms.get(i);  
      a.run();

      int stimulated=0;
      int photonnumber=photons.size();
      int j=0;

      //----------------------------------------------------------------
      // Stimulated absortption/emission
      while(stimulated==0 && j<photonnumber) {
        Photon p=(Photon)photons.get(j);
        if(PVector.dist(a.loc,p.loc)<stimulation_radius) {
          stimulated=1;
          if(a.state==0) {
            a.state=1;
            photons.remove(j);
          }
          else {
            a.state=0;
            addPhoton(new Photon(_parent,new PVector(a.loc.x,a.loc.y), new PVector(p.vel.x,p.vel.y), p.colour));
          }
        }
        j++;
      }

      // ---------------------------------------------------------------
      // Spontanous emission
      if( (a.state==1) && (random(0,1)<decay_propability)) {
        a.state=0;
        float phi=random(0,TWO_PI);
        int cidx=(int)(phi*idx_scale);
        addPhoton(new Photon(_parent, new PVector(a.loc.x,a.loc.y), new PVector(coslookup[cidx],sinlookup[cidx]), 0));
      }
    }

    //-----------------------------------------------------------------------------------------------------------------------
    // Run photons, remove photons and borders and fill detector
    for (int i = 0; i < photons.size(); i++) {
      Photon p = (Photon) photons.get(i);  
      p.run(); 

      if(p.loc.y<sketch_yoffset||p.loc.y>display_height) {
        photons.remove(i);
      }

      else {
        if(p.loc.x<0) {
          photons.remove(i);
        }
        else
        {
          if ((p.loc.x >laser_x2 && p.loc.x<laser_x2+20) && ((p.loc.y >laser_y2-5 && p.loc.y <laser_y2+block+5 ) ||
            (p.loc.y <laser_y1+5 && p.loc.y >laser_y1-block-5 )))
          {
            photons.remove(i);
          }
        }
      }

      //---------------------------------------------------------------------
      //set a photon detector && photons flow into different slot
      if(p.loc.x> detector_x-5) {
        photons.remove(i);
        int idx = (int)((p.loc.y)*precision/(display_height));
        if (idx>0 && idx<precision) {
          slot[idx]=slot[idx]+1;
          photons_sum += slot[idx];
          if (slot[idx]>slotmax) {
            slotmax=slot[idx];
          }
        }
      }
    }
  }


  void addAtom(Atom a) {
    if(!paused) {
      atoms.add(a);
    }
  }


  void addPhoton(Photon p) {
    if(!paused) {
      photons.add(p);
    }
  }


  void pump() {
    for (int i=0;i<atoms.size();i++) {
      if(random(0,1)<pl) {
        Atom a = (Atom) atoms.get(i);
        a.state=1;
      }
    }
  }

  void setLaserPump(float Pump_laser) {
    pl=Pump_laser;
  }
}















