/**
 *  Copyright notice
 *  
 *  This file is part of the Processing sketch `Stimulated Emission' 
 *  http://www.gwoptics.org/processing/stimulated_emission/
 *
 *  Copyright (C) 2009 onwards Mengyao Wang and Andreas Freise
 *  based on an idea by Emil Schreiber.
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
 *  MA  02110-1301, USA.
 */

public class Photon {
  PVector loc;
  PVector vel;
  float colour;
  PApplet _parent;
  int inside;

  Photon(PApplet parent, PVector l, PVector v, float c) { 
    _parent=parent;
    loc = l;
    vel = v;
    colour=c;
    inside=1;
  }

  void run() {
    update();
    borders();
    render();
  }

  void update() {
    loc.add(vel);
  }

  void render() {
    _parent.pushMatrix();
    _parent.fill((201*colour)%255,(72*colour)%255,(913*colour)%255);
    _parent.stroke(222,20,0);
    _parent.ellipse(loc.x,loc.y,4,4);
    _parent.popMatrix();
  }

  void borders() {
    if ((loc.x<laser_x1-2)  && (loc.y>laser_y1) && (loc.y<laser_y2))
    {    
      reflect();
    }

    if ((inside==1) && (loc.x>laser_x2+2)  && (loc.y>laser_y1) && (loc.y<laser_y2))
    {      
      if(random(1)<emSketch.getM2Reflectivity()) {
        reflect();
      }
      else {
        inside=0;
      }
    }
  }


  // reflect photons from curved mirrors mirror
  void reflect()
  {    
    vel.x*=-1;
    // computing deviation from center
    float offset=(loc.y-(laser_y1+(laser_y2-laser_y1)/2));
    offset=abs(offset)*offset/(laser_y2-laser_y1);
    vel.y=vel.y-emSketch.curvature*offset;
  }
}





