/**
 *  Copyright notice
 *  
 *  This file is part of the Processing sketch `Stimulated Emission' 
 *  http://www.gwoptics.org/processing/stimulated_emission/
 *
 *  Copyright (C) 2009 onwards Mengyao Wang and Andreas Freise
 *  based on an idea by Emil Schreiber.
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
 *  MA  02110-1301, USA.
 */

import guicomponents.*;
import org.gwoptics.ValueType;
import org.gwoptics.Logo;
import org.gwoptics.LogoSize;
import org.gwoptics.graphics.GWColour;

//import fullscreen.*; 
//FullScreen fs;

GWSlider reflectivity_m2_slider;
GWSlider pump_laser_slider;
GWSlider mirror_curvature_slider;

GLabel reflectivity_m2_label;
GLabel pump_laser_label; 
GLabel mirror_curvature_label;
GLabel number_photon_label;

GCheckbox pause_chk;
boolean paused=false;

GButton redrawing_distribution;
GButton btnReset;

GTextField reflectivity_m2_text, pump_laser_text, mirror_curvature_text;

Logo gw_logo;

int display_width=880;
int display_height=300;

// temporary variables for distribution plotting
int dist_xoffset;
int dist_yoffset;
float dsc1;
float csc1;
int avg_length=2;

float screenmax =106;
float slotmax = 1;

Laser laser;
EmbeddedSketch emSketch;

int sketch_xoffset=10;
int sketch_yoffset=10;
int slider_length=200;
int label_length=130;
int figure_width=400;
int figure_height=200;
int laser_width=80;
int laser_x1=50;
int laser_y1=100;
int laser_x2=350;
int laser_y2=200;
int laser_height=30;
int block=40;
int detector_width=20;
int detector_x=display_width-detector_width;
int photons_sum=0;
int times=1000;
int s;

// lookup for sinus values
int nlookup=10000;
float [] coslookup = new float[nlookup];
float [] sinlookup = new float[nlookup];
float cosamplitude=10;
float idx_scale=nlookup/TWO_PI;

int  precision = 100;
PImage Logo;
PFont font;

// Default slider values
float default_pump=0.20;
float m2_default_r=0.00;
float default_curvature=0.08;

public float [] slot;

public void setup() {  
  size(900,550);
  frameRate(30);
  // enter fullscreen mode
  //fs = new FullScreen(this);   
  //fs.enter(); 
  //fs.setResolution(1024,640);

  // pre-compute coordinates for distributions drawing
  dist_xoffset=width-figure_width-sketch_xoffset;
  dist_yoffset=height-figure_height-20;
  dsc1=(figure_width-40)/(precision-2*avg_length);
  // The following lines can be used to print the list of available fonts
  //String[] fontList = PFont.list();
  //println(fontList);

  emSketch = new EmbeddedSketch(this, display_width, display_height, precision);                                      
  emSketch.init(); 
  this.setLayout(null); 
  emSketch.setBounds(sketch_xoffset, sketch_yoffset, emSketch.width, emSketch.height); 
  this.add(emSketch); //this adds the embedded sketch to the main sketch

    font = createFont("LucidaSans",12,true);

  //sliders
  reflectivity_m2_slider = new GWSlider(this,"blue18px",sketch_xoffset+label_length,display_height+70,slider_length);
  reflectivity_m2_slider.setValueType(GWSlider.DECIMAL);
  reflectivity_m2_slider.setLimits(m2_default_r, 0f, 1.0f);
  reflectivity_m2_slider.setRenderValueLabel(false);

  pump_laser_slider = new GWSlider(this,"blue18px",sketch_xoffset+label_length,display_height+120,slider_length);
  pump_laser_slider.setValueType(GWSlider.DECIMAL);
  pump_laser_slider.setLimits(default_pump, 0f, 1.0f); 
  pump_laser_slider.setRenderValueLabel(false);	//hides value label

  mirror_curvature_slider = new GWSlider(this,"blue18px",sketch_xoffset+label_length,display_height+170,slider_length);
  mirror_curvature_slider.setValueType(GWSlider.DECIMAL);
  mirror_curvature_slider.setLimits(default_curvature, -.18f, .18f); 
  mirror_curvature_slider.setRenderValueLabel(false);	//hides value label

  //Label
  reflectivity_m2_label = new GLabel(this, "Reflectivity", sketch_xoffset,display_height+65,label_length);
  pump_laser_label = new GLabel(this, "Pump Field", sketch_xoffset,display_height+115,label_length);
  mirror_curvature_label = new GLabel(this, "Mirror Curvature", sketch_xoffset,display_height+165,label_length);

  //Pause option  
  pause_chk = new GCheckbox(this,"Pause",sketch_xoffset,display_height+25,100);
  pause_chk.setSelected(false);

  //Text box  
  reflectivity_m2_text = new GTextField(this,String.valueOf(m2_default_r),sketch_xoffset+label_length+slider_length+20,display_height+65,50,12);
  pump_laser_text = new GTextField(this,String.valueOf(default_pump),sketch_xoffset+label_length+slider_length+20,display_height+115,50,12);
  mirror_curvature_text = new GTextField(this,String.valueOf(default_curvature),sketch_xoffset+label_length+slider_length+20,display_height+165,50,12);


  //logo
  gw_logo = new Logo(this,sketch_xoffset+90,height-40,true,LogoSize.Size35);


  //Button

  GWColour c1,c2,c3;
  c1 = new GWColour(220/255f,220/255f,220/255f);
  c2 = new GWColour(230/255f,230/255f,230/255f);
  c3 = new GWColour(200/255f,200/255f,200/255f);
  int ic1=c1.toInt();
  int ic2=c2.toInt();
  int ic3=c3.toInt();

  btnReset = new GButton(this,"Reset",sketch_xoffset+100,display_height+23,50,20);

  btnReset.setColours(ic1,ic2,ic3);

  redrawing_distribution = new GButton(this,"Redraw",sketch_xoffset+200,display_height+23,50,20);
  redrawing_distribution.setColours(ic1,ic2,ic3);

  emSketch.setM2Reflectivity(m2_default_r);
  emSketch.setLaserPump(default_pump);
  emSketch.setCurvature(default_curvature);

  slot = new float[precision];
}

void draw() {
  background(215);
  emSketch.redraw(); 
  gw_logo.draw(); 
  draw_distribution();
}


//-----------------------------------------------------------------------------------------------
// Draw photon detector distribution 
void draw_distribution() {

  //black figure background 
  fill(2);
  noStroke();
  rect(dist_xoffset, dist_yoffset, figure_width, figure_height);

  //text "photons distribution"
  fill(255);
  textFont(font,12);
  textAlign(LEFT);
  text("Photon Distribution",dist_xoffset+10, dist_yoffset+18);
  fill(200);
  text("Detected Photons/1000 = ",dist_xoffset+10, dist_yoffset+figure_height-10);
  int s=photons_sum/times;
  String t = nf(s,1);
  text(t,dist_xoffset+250, dist_yoffset+figure_height-10);
  stroke(0, 255, 0);

  //Photons number in different slots representing as changing amplitudes and filling coloured slim rectangle
  float dsc2=-screenmax/slotmax;
  float csc2=255/sqrt(slotmax);

  noStroke();
  for (int i=avg_length; i<precision-avg_length; i++) {
    float savg=0.0;
    for (int j=-avg_length;j<=avg_length;j++) {
      savg=savg+slot[i+j];
    }
    savg=savg/(2*avg_length+1);
    fill(sqrt(savg+0.1)*csc2);
    rect(i*dsc1+40+dist_xoffset, height-60, dsc1, dsc2*slot[i]);
  }
}

void handleSliderEvents(GSlider slider) {
  if(slider == reflectivity_m2_slider) {
    emSketch.setM2Reflectivity(slider.getValuef());
    float prec = 100;
    float reflectivity_m2 = slider.getValuef();
    reflectivity_m2_text.setText(String.valueOf(round(prec*reflectivity_m2)/prec));
  }
  else if(slider == pump_laser_slider) {
    emSketch.setLaserPump(slider.getValuef());
    float prec = 100;
    float pump_laser = slider.getValuef();
    pump_laser_text.setText(String.valueOf(round(prec*pump_laser)/prec));
  }
  else if(slider == mirror_curvature_slider) {
    emSketch.setCurvature(slider.getValuef());
    float prec = 100;
    float mirror_curvature = slider.getValuef();
    mirror_curvature_text.setText(String.valueOf(round(prec*mirror_curvature)/prec));
  }
}



void handleCheckboxEvents(GCheckbox checkbox) {
  paused=pause_chk.isSelected();
  if(paused) {
    pause();
  }
  else {
    unpause();
  }
}


void pause() {
  paused=true;
  emSketch.em_pause();
  pause_chk.setSelected(true);
}

void unpause() {
  paused=false;
  emSketch.em_play();      
  pause_chk.setSelected(false);
}


public void keyPressed() {		
  if(key == ' ') {
    if (paused==true) {
      unpause();
    }   
    else {
      pause();
    }
  }
}

public void handleButtonEvents(GButton button) {
  if(button.eventType == GComponent.CLICKED) {
    if(button == btnReset) {
      for (int i=0; i<precision; i++) {
        slot[i]=0;
      }
      photons_sum=0;
      slotmax=1;
      emSketch.setup();

      reflectivity_m2_slider.setValue(m2_default_r);
      pump_laser_slider.setValue(default_pump); 
      mirror_curvature_slider.setValue(default_curvature); 

      reflectivity_m2_text.setText(String.valueOf(m2_default_r));
      pump_laser_text.setText(String.valueOf(default_pump));
      mirror_curvature_text.setText(String.valueOf(default_curvature));

      emSketch.setM2Reflectivity(m2_default_r);
      emSketch.setLaserPump(default_pump);
      emSketch.setCurvature(default_curvature);
    }
    else if (button== redrawing_distribution) {
      for (int i=0; i<precision; i++) {
        slot[i]=0;
      }
      photons_sum=0;
      slotmax=1;
    }
  }
}


void handleTextFieldEvents(GTextField tfield) {
  String line1="";
  float temp_val;
  if(tfield == reflectivity_m2_text) {
    switch(reflectivity_m2_text.eventType) {
    case GTextField.CHANGED:
      break;
    case GTextField.SET:
      break;
    case GTextField.ENTERED:
      line1 = reflectivity_m2_text.getText();
      temp_val = parseFloat(line1);
      reflectivity_m2_slider.setValue(temp_val);
      emSketch.setM2Reflectivity(temp_val);
      break;
    }
  }
  else if(tfield ==  pump_laser_text) {
    switch( pump_laser_text.eventType) {
    case GTextField.CHANGED:
      break;
    case GTextField.SET:
      break;
    case GTextField.ENTERED:
      line1 =  pump_laser_text.getText();
      temp_val = parseFloat(line1);
      pump_laser_slider.setValue(temp_val);
      emSketch.setLaserPump(temp_val);
      break;
    }
  }
  else if(tfield ==  mirror_curvature_text) {  
    switch( mirror_curvature_text.eventType) {
    case GTextField.CHANGED:
      break;
    case GTextField.SET:
      break;
    case GTextField.ENTERED:
      line1 =  mirror_curvature_text.getText();
      temp_val = parseFloat(line1);
      mirror_curvature_slider.setValue(temp_val);
      emSketch.setCurvature(temp_val);
      break;
    }
  }
}
/*
  static public void main(String args[]) {
 // use this if fullscreen library is NOT used:
 PApplet.main(new String[] { 
 "--present", "--bgcolor=#666666", "--hide-stop","StimulatedEmission"});
 // use the following line if the fullscreen library is used:
 //PApplet.main(new String[] {
 //  "StimulatedEmission"    });
 }
 
 */





















